!!$
!!$=head1 NAME
!!$
!!$ bwi-wet3d-cgstep.x - 2/3D one-way wave-equation migration velocity analysis
!!$   Written by Yaxun Tang
!!$   tang@sep.stanford.edu
!!$   Version 1.0
!!$   June, 2011
!!$
!!$=head2 SYNOPSIS FOR WAVEFIELD TOMOGRAPHY
!!$
!!$   bwi-wet3d-cgstep.x mode=tominv parameters
!!$
!!$=head3 PARAMETERS
!!$
!!$
!!$ imsk=IMSK.H(input, optional), image mask, useful when imgmsk=y
!!$      sep77 hypercube where n1=x n2=y n3=z
!!$      for 2-D, set n2=1 o2=0 d2=1
!!$      the dimensions and samplings of x, y and z should be the same as those of BIMG.H 
!!$
!!$ bimg=BIMG.H(output), a temporary image file, constantly overwritten by background image and perturbed image at each iteration
!!$      sep77 hypercube where n1=subsurface offset x n2=subsurface offset y n3=x n4=y n5=z
!!$
!!$
!!$
!!$ stkimg=[y]/n, if the DIMG.H input for back-projection hasn't been stacked over source axis, set stkimg=y, otherwise n
!!$
!!$ hcut=[0], the truncation point for offset-domain DSO, useful when objtyp=offdso or combos
!!$ 
!!$ dsotyp=[1]/2, DSO type, useful when objtyp=offdso or combos
!!$               1 for |h|
!!$               2 for a Gaussian taper function
!!$
!!$ dsorms=y/[n], whether or not to noramlize DSO with an image RMS amplitudes, useful when objtyp=offdso or combos
!!$
!!$ objtyp=[imgpow]/offdso/combos, type of objective function
!!$                                imgpow - image stack power maximization, in this case, set nhx=1 ohx=0 dhx=1 nhy=1 ohy=0 dhy=1
!!$                                offdso - subsurface-offset-domain DSO
!!$                                combos - combination of image stack power maximization and DSO
!!$
!!$ objeps=[1.], trade-off parameter to adjust the relative importance of DSO and image stack power (ISP)
!!$              Fobj = Fobj_DSO + objeps*Fobj_ISP, useful when objtyp=combos
!!$
!!$
!!$
!!$=head1 COMMENTS
!!$
!!$ Report bugs to Yaxun Tang, tang@sep.stanford.edu
!!$
!!$=cut

program main
  use bwi_restom_mod
  use bwi_kernel_mod
  use  base_ioutil_mod
  use  bwi_struct_mod
  implicit none

  type(dat_grid)                                  :: dgrid
  type(pro_grid)                                  :: pgrid
  type(vel_grid)                                  :: vgrid
  type(q_grid)                                    :: qgrid
  type(img_grid)                                  :: igrid
  type(hes_grid)                                  :: hgrid
  type(bwi_ftag)                                  :: ftag_ 
  type(axis)                                      :: aiter, adip
  type(axis)                                      :: axk, ayk, azk
  real           , dimension(:)     , allocatable :: dfcx0, dfcy0
  integer        , dimension(:,:,:) , allocatable :: selx, sely
  integer                                         :: stat, iter, niter, nsrc, dsotyp, dipitr, siter, na, idip
  integer                                         :: rectx_grd, recty_grd, rectz_grd, nsmt_grd
  integer                                         :: rectx_dip, recty_dip, rectz_dip, nsmt_dip
  real                                            :: xk_min, yk_min, zk_min
  real                                            :: xk_max, yk_max, zk_max
  real                                            :: alph, hcut, objeps, dipeps
  real                                            :: dip_min, dip_max  
  double precision, dimension(:)    , allocatable :: fobj
  real            , dimension(:)    , allocatable :: vmod
  real            , dimension(:,:,:), allocatable :: dipx, dipy
  character(len=6)                                :: objtyp, method, smttyp
  character(len=5)                                :: scheme 
  logical                                         :: wntdip, smtgrd, modmov, objmov, switch, dsorms, grdfix
  logical, parameter                              :: T = .true., F = .false.
  character(len=256)                              :: fobj_, mmov_, dipx_, dipy_


  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()
  call sep_begin_prog()


  call from_param("hcut"  , hcut  , 20.     )
  call from_param("dsotyp", dsotyp, 1       )
  call from_param("dsorms", dsorms, .false. )
  call from_param("objtyp", objtyp, "imgpow") 
  call from_param("switch", switch, .false. )
  call from_param("scheme", scheme, "UW2x3" )
  call from_param("objeps", objeps, 1.      ) 
  call from_param("objmov", objmov, .false. )
  call from_param("grdfix", grdfix, .false. )




  allocate(fobj(1))
  fobj = dble(0.)
 write(0,*)"initiation"

    ftag_%bimg_ = "bimg"       !!.. bakground image
    call auxinout(ftag_%bimg_)
    !   read image axeses
    !   -----------------
    call hinaxis(ftag_%bimg_, igrid%ahx, 1)
    call hinaxis(ftag_%bimg_, igrid%ahy, 2)
    call hinaxis(ftag_%bimg_, igrid%ax , 3)
    call hinaxis(ftag_%bimg_, igrid%ay , 4)
    call hinaxis(ftag_%bimg_, igrid%az , 5)

  
   write(0,*) "igrid%ahx, igrid%ahy,igrid%ax, igrid%ay, igrid%az",igrid%ahx, igrid%ahy,igrid%ax, igrid%ay, igrid%az

    igrid%image_jx = 1; igrid%image_dx = igrid%image_jx*igrid%ax%d
    igrid%image_jy = 1; igrid%image_dy = igrid%image_jy*igrid%ay%d
    igrid%image_jz = 1; igrid%image_dz = igrid%image_jz*igrid%az%d

       igrid%asx%n=1; igrid%asx%o=0.; igrid%asx%d=1.
       igrid%asy%n=1; igrid%asy%o=0.; igrid%asy%d=1.


! compute the residual and objective function with initial model
! --------------------------------------------------------------
  write(0,*) "compute the image residual and objective function value"
  stat = bwi_restom_oper(ftag_%bimg_, ftag_%hess_, ftag_%imsk_, igrid, hgrid, fobj(1), hcut, dsotyp, dsorms, objtyp, objeps, grdfix)

  call auxputch("n1", "i", 1, 'fobj')
  call sep_write(real(fobj), 'fobj')
  write(0,*)"fobj", fobj


  write(0,*)"***********************"
  write(0,*)"      Finished!!!      "
  write(0,*)"***********************"

  call sep_end_prog()

  call exit()

end program
